import sys
import os
import time
import datetime
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
from dateutil.relativedelta import *
import numpy
from common import *

class Data:
    """This classe define the set of data for the observation or model
    on which will be calculate the statistics.
    The variables are the following:
    - datanames: names of experiments or observational set (string)
    - varname: name of the variables on which the statitics must be calculated (string)
    - inputdir: directory where are stored the original data (string)
    - outpudir: directory to write the output data (string)
    - filelist: list of file on which the calculations must be done (list of string)
    - sdates: list of startdate associated to each file (list of date)
    - filetype: type of the files in Filelist (string)
    - freq: frequency between 2 timesteps (timedelta)
    - nmenber: number of member (int)"""
    def __init__(self, dataname, varname):
        self._dataname=dataname
        self._varname=varname
        self._inputdir=""
        self._outputdir=""
        self._filelist=[]
        self._sdates=[]
        self._filetype=""
        self._freq=-1
        self._nmenber=-1
    
    
    
    ######## set ########

    def set_dataname(self, dataname):
        self._dataname=dataname

    def set_varname(self, varname):
        self._varname=varname
   
    def set_inputdir(self, inputdir):
        self._inputdir=inputdir

    def set_outputdir(self, outputdir):
        self._outputdir=outputdir

    def set_filelist(self, filelist):
        self._filelist=filelist

    def set_sdates(self, sdates):
        self._sdates=sdates

    def set_filetype(self, filetype):
        self._filetype=filetype

    def set_freq(self,freq):
        self._freq=freq

    def set_nmember(self, nmenber):
        self._nmenber = nmenber

    ######## get ########

    def get_dataname(self):
        return self._dataname

    def get_varname(self):
        return self._varname
   
    def get_inputdir(self):
        return self._inputdir

    def get_outputdir(self):
        return self._outputdir

    def get_filelist(self):
        return self._filelist

    def get_sdates(self):
        return self._sdates

    def get_filetype(self):
        return self._filetype

    def get_freq(self):
        return self._freq

    def get_nmember(self):
        return self._nmenber
    ######## print ########

    def see(self):
        print "dataname: ", self.get_dataname()
        print "varname: ", self.get_varname()
        print "inputdir: ", self.get_inputdir()
        print "outputdir: ", self.get_outputdir()
        print "filelist: ", self.get_filelist()
        print "sdates: ", self.get_sdates()
        print "filetype: ", self.get_filetype()
        print "freq: ", self.get_freq()
        print "nmember: ", self.get_nmember()

    ######## set_outputdirAuto ########

    def set_outputdirAuto(self):
        """look for input and output directory corresponding to the dataname"""
        inputdir=self.get_inputdir()
        if "/" in  inputdir:
            #remove the last part of the input directory and replace it by statistics
            outputdir="/".join(inputdir.split("/")[:-3])+"/monthly_statistics_new/"
        else:
            print "The path specify must be an abolute path"
            print "this path is not correct or is empty:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
            print "Created directory:"
            print outputdir


   ######## select_leadtime ########

    def select_leadtime(self, leadtime1, leadtime2):
        """select the time period between leadtime1 and leadtime2
        in all files of the DataMod object:
        -leadtime1: in month (relativedelta)
        -leadtime2: in month (relativedelta)
        the file must have the correct format for cdo
        (run the format_files_nc method in case of failure)
        (to be changed when the right named convention has been set)
        """
        cdo=Cdo()
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        #timedelta can not be compared: find a solution
        #if ((leadtime1!=relativedelta()) and (leadtime2 < leadtime1)):
        #        print leadtime1
        #        print leadtime2
        #        print "the second leadtime must be larger than the first one"
        #        sys.exit(0)

        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            
            fdate1=d+leadtime1
            fdate2=d+leadtime2
            

            print "the selected time period is:"
            print fdate1, fdate2

            #calculate the last date of the file
            lastdate=get_last_date(f)

            #check if the time period is smaller the the length of the file
           
            if (fdate2 > lastdate):
                print "the selected time period is not include in the file:"
                print f
                sys.exit(0)


            fileout=(f.split("/")[-1])
            lead1str=str(leadtime1.months).zfill(2)
            lead2str=str(leadtime2.months).zfill(2)
            fileout=fileout.split(".")[0]+"_"+lead1str+"-"+lead2str+".nc"
            cdo.seldate(fdate1,fdate2, input=f, output=fileout, options="-r -f nc")
            filelist[i]=fileout
            sdates[i]=fdate1
            

        exp_season=copy.deepcopy(self)
        exp_season.set_filelist(filelist)
        exp_season.set_sdates(sdates)
        exp_season.set_varname(varname + "_" + lead1str + "-" + lead2str)
        return exp_season






    ###################################
    ########    STATISTICS     ########
    ###################################     


    ######## monthly_percentile ########
 
    def monthly_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.monpctl(percentile, input=(f,
                                           cdo.monmin(input=f,options="-r -f nc"),
                                           cdo.monmax(input=f,options="-r -f nc")), 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout
            #when ncdf4 instaled change name of variables in files


        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelist)
        exp_stat.set_freq = relativedelta(months=+1)
        exp_stat.set_varname(varname + "_q" + str(percentile))
       
        return exp_stat

     ######## monthly_nbday ########

    def monthly_nbday(self, percentile, comp):
        listmondata=self.create_monthly_list()
        listdatastat=[]
        for data in listmondata:
            datastat=data.seasonal_nbday(percentile, comp)
            listdatastat.append(datastat)

        var0=(listdatastat[0]).get_varname()
        var0=quit_leadtime(var0)
        var0=var0.split("-")
        var1=(listdatastat[-1]).get_varname()
        var1=quit_leadtime(var1)
        var1=var1.split("-")

        varnew=var0[0]+"-"+var1[1]
        expstat=concat_monthly_data(listdatastat,
                                    varnew)
        return expstat

     ######## seasonal_nbday ########


    def seasonal_nbday(self, percentile, comp):
        """calculate the monthly percentile for all file in filelist"""
        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        
        #calculate the climatological percentile
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                sys.exit(0)
        merge=cdo.mergetime(input=filelist, options="-O -r -f nc")
        
        
        #open merge file
        ncdfmer=NetCDFFile(merge, "r")
        variables=ncdfmer.variables.keys()
        allDimNames = ncdfmer.dimensions.keys() 

            
        if varname in variables:
            varmer = ncdfmer.variables[varname]
            varaux=varname
        else:
            varaux=varname.split("_")[0]
            if varaux in variables:
                varmer = ncdfmer.variables[varaux]
            else:
                print "the variables %s or %s is not found in the file:" %varname,varaux
                print merge
                sys.exit(0)

        shapmer=shape(varmer)
        if not(len(shapmer) == 4):
            if (len(shapmer) == 3):
                varmer=numpy.resize(varmer,(shapmer[0],1,shapmer[1],shapmer[2]))
                shapmer=shape(varmer)
            else:
                print "the merged file does not have 3 or 4 dimensions"
                sys.exit(0)
        if not(shapmer[1] ==  nmember):
            print "the number of member in the merged file is not correct"
            sys.exit(0)

        print "the merge of file has been done successfully"
        varmer=numpy.resize(varmer,(shapmer[0]*shapmer[1],shapmer[2],shapmer[3]))
        percent=numpy.percentile(varmer, percentile, axis=0)
        filelistout=[]
        for f in filelist:
            print "treating file:", f
            ncdfin=NetCDFFile(f, "r")
            varin=ncdfin.variables[varaux]
            shapin=shape(varin)
            leadtime=shapin[0]
            if not(len(shapin) == 4):
                if not(len(shapin) == 4):
                    if (len(shapin) == 3):
                        varin=numpy.resize(varin,(shapin[0],1,shapin[1],shapin[2]))
                        shapin=shape(varin)
                    else:
                        print "the file does not have 3 or 4 dimensions"
                        sys.exit(0)
            if not(shapin[1] ==  nmember):
                print "the number of member in the file is not correct"
                sys.exit(0)

            varmercomp=numpy.array([percent,]*nmember)
            varout=numpy.zeros((shapmer[1],shapmer[2],shapmer[3]), dtype='f')
            
            for t in range(leadtime):
                if comp == "greater":
                    varout=varout+numpy.greater_equal(varin[t,:,:,:],varmercomp)
                if comp == "less":
                    varout=varout+numpy.less_equal(varin[t,:,:,:],varmercomp)
            
            varout=varout/leadtime
            if numpy.amax(varout) > 1:
                print "the percent of day is greater than 1"
                print shape(varout)
                print leadtime
                sys.exit(0)
            #convert the output in a standart python type
            fileout=f.split("/")[-1]
            fileout=fileout.split(".")[0]+"_nbdays.nc"
            ncdfout=NetCDFFile(fileout, "w")
            
            write_lonlat(ncdfout,ncdfin)
            
            ncdfout.createDimension('time', None)
            if "reftime" in ncdfin.variables.keys():
                timevar="reftime"
            elif "time" in ncdfin.variables.keys():
                timevar="time"
            else:
                print "no time variable found"
                sys.exit(0)
            #create the reftime variable
            reftime=ncdfout.createVariable(timevar, "i", ('time',))
            reftime[:] = [0]
            reftimein = ncdfin.variables[timevar]
            reftime.units = reftimein.units
            reftime.standard_name = "forecast_reference_time"
            reftime.longname = "forecast reference time"
            reftime.calendar=reftimein.calendar
    
            ncdfout.createDimension('ensemble', nmember)
            
            var=ncdfout.createVariable(varname, 'f', ("time", "ensemble", "latitude", "longitude",))
            var[:]=varout
            ncdfout.close()
            ncdfin.close()
            filelistout.append(fileout)
        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelistout)
        exp_stat.set_freq = relativedelta(months=+1)

        #get last date of the last file
        dateend=get_last_date(filelist[-1])
        period=str(sdates[0]).replace("-","")+"-"+str(dateend).replace("-","")

        #to be changed when final convention defined!!!!!
        exp_stat.set_varname(varname + "_nbdays_q" + str(percentile) + "_" + period)
        return exp_stat

     ######## seasonal_percentile ########


    def seasonal_percentile(self, percentile):
        """calculate the monthly percentile over the whole period
        for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.timpctl(percentile, input=(f,
                                           cdo.timmin(input=f,options="-r -f nc"),
                                           cdo.timmax(input=f,options="-r -f nc")), 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout

        
        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelist)
        exp_stat.set_freq = relativedelta(months=+1)
        #to be changed when final convention defined!!!!!
        exp_stat.set_varname(varname + "_q" + str(percentile) )
       
        return exp_stat
