import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *
from nco import Nco


class DataEC23(DataMod):
     """ classe of data for the b0cm and b0gq experiments
     """
    def __init__(self, dataname, varname):
        DataMod.__init__(self, dataname, varname)
        self._nmenber=10
        self._freq=datetime.timedelta(0, 0, 0, 0, 0, 6)

    def set_inputdirAuto(self):
        """defining the path to look for the model daily data"""
         dataname=self.get_dataname()
       
         inputdir = "/cfunas/exp/ecearth/"+dataname+"/daily_grb/"+  varname + "_6hourly/"

         if (not os.path.isdir(inputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)

        self.set_inputdir(inputdir)


   def format_files_nc(self):
        """transform the grb file 
        into ncdf file readable for cdo"""
        
        cdo=Cdo()
        nco = Nco()

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        nmember=self.get_nmember()
        freq=self.qet_freq(freq)


        #in b0cm and b0gq the leadtime is 4month
        #hard coded
        nbmon=relativedelta(months=+4, days=-1)


        for i in range(len(filelist)):
            f=filelist[i]
            d1=sdates[i]
            
            #calculate the last date of the file
            d2=d1+nbmon
            
            #calculate the number of timestep
            nbstep=((d2-d1).days)*86400/freq.seconds

            #set the grid
            f=cdo.setgridtype("regular", input=f)
            f=cdo.copy( input=f, option="-r -t ecmwf -f nc" )

            #list to store the members
            lstmemb=[]
            #loop over the member
            for (imemb in range(10)):
                
                #select one member
                fmemb=nco.ncks(input=f, options='-O -d time,%i,,%i' % (imemb,nmember))                
                fmemb=nco.ncks(input=fmemb, options='-O -d time,0,%i' % nbstep)
                fmemb=cdo.settaxis(d, "12:00", freq, input=fmemb, options="-r -f nc")
                lstmemb.append(fmemb)
            f=nco.ncecat(input=lstmemb, options="-h -O")
            f=nco.ncpdq(input=f, options="-O -h -a time,record")
