from data import Data
#import pandas as pd
import datetime
import time
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *

class DataMod(Data):
    """ classe of data for the model
    """

    def set_files(self, date1, date2):
        """create the Filelist and sdates argument with all files
        between date1 and date2 separated by one year
        check if these files exist
        check their type"""
        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]

        filename=self.get_varname() + "_" + str(date1).replace("-","")
        #we determine the type of file with the first year
        try:
            filename=glob.glob(inputdir+filename+'.nc')
        except:
            try:
                filename=glob.glob(inputdir+filename+".grb")
            except:
                print "the file %s does not exist" % (inputdir+filename+".nc")
                print "and the file %s does not exist" % (inputdir+filename+".grb")
                print "the directory %s must contain .nc or .grb file for the startdate %s" % (inputdir, str(date1))
                sys.exit(0)
            filetype="grb"
        filetype="nc"
        filelist.append(filename[0])
        datelist.append(date1)
        date1 = date1 + relativedelta(years=+1)
        d=date1
        while d <= date2:
            filename=self.get_varname() + "_" + str(d).replace("-","")
            try:
                filename=glob.glob(inputdir+"/"+filename+"."+filetype)
            except:
                print "the file %s does not exit" % inputdir+filename+"*."+filetype
                print "while the file %s exists"  % inputdir+filelist[1]
                print "all file must have the same type (grb or nc)"
                sys.exit(0)
            datelist.append(d)
            filelist.append(filename[0])
            d = d + relativedelta(years=+1)

        self.set_filelist(filelist)
        self.set_sdates(datelist)


    def create_monthly_list(self):
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        freq=self.get_freq()
        
        f=filelist[0]
        lastdate=get_last_date(f)
        nbmon=diff_month(lastdate,sdates[0])
        listmondata=[]
        for imon in range(nbmon+1):
            leadtime1=relativedelta(months=+imon)
            leadtime2=relativedelta(months=+imon+1, days=-1)
            listmondata.append(self.select_leadtime(leadtime1, leadtime2))
        return listmondata



    #### to be simplified!!!!!!!!!!!!!!!!!!!!!!!
    def write_output(self, units, unit_long, 
                     long_name):
        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        #create output directory
        outputdir=self.get_outputdir()+"/"+varname+"/"        
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
        

        #loop over all file
        for i in range(len(filelist)):
            f=filelist[i]
            ftemp=(f.split("/")[-1])
            ftemp=ftemp.split(".")[0]+"_temp.nc"

            #open input and output
            ncdfin=NetCDFFile(f, "r")
            ncdfout=NetCDFFile(ftemp, "w")

            variables=ncdfin.variables.keys()
            allDimNames = ncdfin.dimensions.keys() 
  
            ##### time ######
            write_time(ncdfout, ncdfin)
           
            ##### longitude - latitude ######
            write_lonlat(ncdfout, ncdfin)

            ##### ensembles ######
            ncdfout.createDimension('ensemble', nmember)

            #### variable #####
            var=ncdfout.createVariable(varname, 'f', ("time", "ensemble", "latitude", "longitude",))

             #temporary solution! very bad!
            #to be change!!!!!!!!
            #look for a variable named varname or which is the beginning of varname
            #   #add a case for reconstructed variables (prlr or sfcwind) 
            #   #or find a way to change the name of the variable
            if varname in variables:
                varin = ncdfin.variables[varname]
                varaux=varname
            else:
                varaux=(varname.split("_"))[0]
                if varaux in variables:
                    varin = ncdfin.variables[varaux]
                else:
                    varaux3=""
                    for varaux2 in variables:
                        if varaux2.startswith(varaux):
                            varaux3=varaux2
                    if varaux3 == "":
                        print "there is no variable named %s or %s or starting by %s in the file" % (varname, varaux, varaux)
                        print f
                        print variables
                        sys.exit(0)
                    else:
                        varaux=varaux3
                        varin = ncdfin.variables[varaux]

            var.data_type = "float" 
            var.units = units
            var.unit_long = unit_long
            var.standard_name = varname 
            var.long_name = long_name
            
            var[:]=varin

            ##### institution ######
            write_insti(ncdfout,nmember,dataname)
            
            ###### experiment_id ######
            write_id(ncdfout,nmember,dataname)
           
            ###### realization ######
            write_realization(ncdfout,nmember)

            ###### source ######
            write_source(ncdfout,nmember,dataname)
            ncdfout.close()
            fileout=varname+"_"+str(sdates[i]).replace("-","")+".nc"
            shutil.copyfile(ftemp, outputdir+fileout)

            print "file saved:"
            print outputdir+fileout



